/* reseni soustavy lin. alg. rovnic Gaussovou eliminacni metodou */
/* zadani rovnice v souboru                                      */
/* zamena sloupcu, vyber sloupce s max. abs. hodnotou            */
/* osetreni na singularitu soustavy                              */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

typedef double *pd;

/* Alokace pameti s osetrenim na nedostatek pameti */
void *myall(unsigned kolik)
{
 void *p;
 if ((p=malloc(kolik))==NULL)
  {
   printf("\nNedostatek pameti!\n");
   exit(1);
  }
 return p;
}


/* Vypis soustavy rovnic */
void vyprov(int n, pd a, pd b)
{
 int i,j;
 pd px;
 for (i=0;i<n;i++)
   {
    px=a+n*i;
    for (j=0;j<n;j++)  printf("%+7.3f*%c  ", px[j],('Z'-n+1+j));
    printf("=%+7.3f\n", b[i]);
   }
}


int gauss(int n, pd a, pd b)
{
 int *p1,*p2;
 int i,j,k,m,*ind;
 int kk, jj;
 pd *mat,pr,px;
 double pom;

 /* Vytvoreni pomocneho pole ukazatelu */
 mat=(pd *)myall(n*sizeof(pd));
 /* Vytvoreni pomocneho pole indexu */
 ind=(int *)myall(n*sizeof(int));
 for (i=0;i<n;i++)
   {
    mat[i]=a+n*i;
    ind[i]=i;
   }

 /* Primy beh */
 for (k=1;k<n;k++)
   {
    /* hledani maxima a zamena sloupcu */
    p1=ind+k-1;
    for (p2=ind+k;p2<ind+n;p2++)
      if (fabs(mat[k-1][*p2])>fabs(mat[k-1][*p1]))
	{
	 m=*p1;
	 *p1=*p2;
	 *p2=m;
	}

    pr=mat[k-1];
    kk=ind[k-1];
    if (pr[kk]==0) return 0;
    for (i=k;i<n;i++)
      {
       px=mat[i];
       pom=px[kk]/pr[kk];
       for (j=k-1;j<n;j++)
	 {
	  jj=ind[j];
	  px[jj]-=pom*pr[jj];
	 }
       b[i]-=pom*b[k-1];
      }
   }

 /* Zpetny beh */
 for (k=n-2;k>=0;k--)
   {
    kk=ind[k+1];
    pr=mat[k+1];
    if (pr[kk]==0.0) return 0;
    for (i=k;i>=0;i--)
     {
      pom=mat[i][kk]/pr[kk];
      for (j=k;j<n;j++)
	{
	 jj=ind[j];
	 mat[i][jj]-=pom*pr[jj];
	}
      b[i]-=pom*b[k+1];
     }
   }

 /* Konecna uprava */
 pr=mat[0];
 for (i=0;i<n;i++)
  {
   pr[i]=b[i]/mat[i][ind[i]];
  }
 for (i=0;i<n;i++)
   b[i]=pr[ind[i]];

 free(ind);
 free(mat);
 return 1;
}

void main()
{
 int rad;
 int i,j;
 pd A, B;
 FILE *fr;
 double X;
 char s[120];

 /* nacteni soustavy rovnic ze souboru */
 printf("\nZadej jmeno souboru s popisem soustavy rovnic: ");
 gets(s);
 fr=fopen(s, "r");
 if (fr==NULL) return;
 fscanf(fr, "%d", &rad);
 if (rad<2) {fclose(fr); return;}


 A=(pd)myall(rad*rad*sizeof(double));
 B=(pd)myall(rad*sizeof(double));

 for (i=0;i<rad;i++)
   {
    for (j=0;j<rad;j++)
      {
      fscanf(fr, "%lf", &X);
      *(A+rad*i+j)=X;
      }
    fscanf(fr, "%lf", B+i);
   }
 fclose(fr);

 printf("\nZadana soustava rovnic\n");
 vyprov(rad, A, B);

 if (gauss(rad, A, B))
  {
   /* vypise reseni soustavy rovnic */
   printf("Reseni\n");
   for (i=0;i<rad;i++)
     printf("%c=%+7.3f  ", 'Z'-rad+1+i, B[i]);
   printf("\n");
   }
 else
   printf("\nSoustava nema reseni\n");

 for (;getchar()!='\n';);

}
